package com.xxh.cloud.web.common.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Component
public class StpUserPermissionCacheService {

    @Autowired
    private RedisUtils redisUtils;

    private String getUserPermissionKey(String userId){
        return StrUtil.format("stp:u:p:{}",userId);
    }

    private String getUserRoleKey(String userId){
        return StrUtil.format("stp:u:r:{}",userId);
    }

    private String getUserDataDeptKey(String userId){
        return StrUtil.format("stp:u:d:{}",userId);
    }


    /**
     * 新增:用户功能权限
     * */
    public void addUserPermission(String userId, Set<String> permissions){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        if(CollUtil.isEmpty(permissions)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户权限不能为空");
        }
        String key=this.getUserPermissionKey(userId);
        redisUtils.set(key, StrJoinSplitUtils.join(permissions),RedisUtils.Day_Thirty_EXPIRE);
    }

    /**
     * 获取
     * */
    public List<String> getUserPermission(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserPermissionKey(userId);
        Object cacheObj= redisUtils.get(key);
        if(cacheObj==null){
            return null;
        }
        return StrJoinSplitUtils.split2List(Convert.toStr(cacheObj));
    }

    /**
     * 移除一个
     * */
    public void removeUserPermission(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserPermissionKey(userId);
        redisUtils.delete(key);
    }

    /**
     * 移除多个
     * */
    public void removeUserPermission(Collection<String> userIds){
        if(CollUtil.isEmpty(userIds)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        Set<String> keys=new HashSet<>();
        userIds.forEach(userId->{
            keys.add(this.getUserPermissionKey(userId));
        });
        redisUtils.delete(keys);
    }


    /**
     * 用户角色
     * */
    public void addUserRole(String userId,Set<String> roles){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        if(CollUtil.isEmpty(roles)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户角色不能为空");
        }
        String key=this.getUserRoleKey(userId);
        redisUtils.set(key, StrJoinSplitUtils.join(roles),RedisUtils.Day_Thirty_EXPIRE);
    }

    public void removeUserRole(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserRoleKey(userId);
        redisUtils.delete(key);
    }

    public List<String> getUserRole(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserRoleKey(userId);
        Object cacheObj= redisUtils.get(key);
        if(cacheObj==null){
            return null;
        }
        return StrJoinSplitUtils.split2List(Convert.toStr(cacheObj));
    }

    public void removeUserRole(Collection<String> userIds){
        if(CollUtil.isEmpty(userIds)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        Set<String> keys=new HashSet<>();
        userIds.forEach(userId->{
            keys.add(this.getUserRoleKey(userId));
        });
        redisUtils.delete(keys);
    }

    /**
     * 用户数据权限(部门ID)
     * */
    public void addUserDataDept(String userId,Set<String> deptIds){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        if(CollUtil.isEmpty(deptIds)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户数据部门不能为空");
        }
        String key=this.getUserDataDeptKey(userId);
        redisUtils.set(key, StrJoinSplitUtils.join(deptIds),RedisUtils.Day_Thirty_EXPIRE);
    }



    public void removeUserDataDept(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserDataDeptKey(userId);
        redisUtils.delete(key);
    }

    public List<String> getUserDataDept(String userId){
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        String key=this.getUserDataDeptKey(userId);
        Object cacheObj= redisUtils.get(key);
        if(cacheObj==null){
            return null;
        }
        return StrJoinSplitUtils.split2List(Convert.toStr(cacheObj));
    }

    public void removeUserDataDept(Collection<String> userIds){
        if(CollUtil.isEmpty(userIds)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户ID不能为空");
        }
        Set<String> keys=new HashSet<>();
        userIds.forEach(userId->{
            keys.add(this.getUserDataDeptKey(userId));
        });
        redisUtils.delete(keys);
    }



}
