package com.xxh.cloud.framework.common.redis;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

public abstract class BasicCacheServiceImpl <T> implements IBasicCacheService<T>{
    @Autowired
    protected RedisUtils redisUtils;

    public abstract String getCacheKey(String id);

    public abstract String getCacheKeyPattern();

    public abstract Class<T> getJsonClazz();

    public abstract String getId(T entity);

    /**
     * DB
     * */
    public abstract T getById2DB(String id);

    /**
     * DB
     * */
    public abstract List<T> getListByIds2DB(Collection<String> ids);

    /**
     * 缓存过期时间，单位：秒 (默认不过期）
     * */
    public long getCacheExpire(){
        return RedisUtils.NOT_EXPIRE;
    }

    @Override
    public T getById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        String key= this.getCacheKey(id);
        Object json= redisUtils.get(key);
        if(json!=null){
            return JSON.parseObject(json.toString(), this.getJsonClazz());
        }

        T data= this.getById2DB(id);
        if(data!=null){
            redisUtils.set(key,JSON.toJSONString(data),getCacheExpire());
        }
        return data;
    }

    @Override
    public Map<String, T> getMapByIds(Collection<String> ids) {
        Map<String, T> data=new HashMap<>();
        if(CollUtil.isEmpty(ids)){
            return data;
        }
        List<T> list = this.getListByIds(ids);
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(this.getId(item),item);
            });
        }
        return data;
    }

    @Override
    public List<T> getListByIds(Collection<String> ids) {
        List<T> data= new ArrayList<>();
        if(CollUtil.isEmpty(ids)){
            return data;
        }
        Set<String> keys=new HashSet<>();
        ids.forEach(id->{
            keys.add(this.getCacheKey(id));
        });

        Map<String, Object> jsonMap=redisUtils.getBatchMap(keys);
        if(MapUtil.isEmpty(jsonMap)){
            data=this.getListByIds2DB(ids);
            this.setCache(data);
            return data;
        }

        Set<String> dbIds=new HashSet<>();
        for(String id:ids){
            Object json= jsonMap.get(this.getCacheKey(id));
            if(json!=null){
                data.add(JSON.parseObject(json.toString(),this.getJsonClazz()));
            }else {
                dbIds.add(id);
            }
        }

        if(dbIds.size()>0){
            List<T> list= this.getListByIds2DB(dbIds);
            if(CollUtil.isNotEmpty(list)){
                data.addAll(list);
                this.setCache(list);
            }
        }
        return data;
    }



    private void setCache(List<T> list){
        if(CollUtil.isEmpty(list)){
            return;
        }
        Map<String, String> map=new HashMap<>();
        for(T obj:list){
            map.put(this.getCacheKey(this.getId(obj)),JSON.toJSONString(obj));
        }
        redisUtils.multiSet(map,this.getCacheExpire());
    }



    @Override
    public void removeById(String id) {
        if(StrUtil.isEmpty(id)){
            return;
        }
        String key= this.getCacheKey(id);
        redisUtils.delete(key);
        this.onRemoveById(id);
    }

    @Override
    public void removeByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return;
        }
        Set<String> keys=new HashSet<>();
        ids.forEach(id->{
            keys.add(this.getCacheKey(id));
        });
        redisUtils.delete(keys);
        this.onRemoveByIds(ids);
    }

    @Override
    public Set<String> getCacheKeySet() {
        String keyPattern=this.getCacheKeyPattern();
        if(StrUtil.isEmpty(keyPattern)){
            return new HashSet<>();
        }
        return redisUtils.keys(keyPattern);
    }

    @Override
    public void removeAll() {
        Set<String> keys= this.getCacheKeySet();
        if(CollUtil.isNotEmpty(keys)){
            redisUtils.delete(keys);
        }
        this.onRemoveAll();
    }


    public void onRemoveAll(){
        //todo:
    }

    public void onRemoveByIds(Collection<String> ids){
        //todo:
    }

    public void onRemoveById(String id) {
        //todo:
    }

}
