/**
 * Copyright (c) 2018 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xxh.cloud.framework.common.redis;

import com.xxh.cloud.framework.common.redis.mq.callback.MultipleMessageListener;
import com.xxh.cloud.framework.common.redis.mq.callback.OneMessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * Redis工具类
 */
@Component
public class RedisUtils {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    /**  默认过期时长为24小时，单位：秒 */
    public final static long DEFAULT_EXPIRE = 60 * 60 * 24L;
    /**  过期时长为1小时，单位：秒 */
    public final static long HOUR_ONE_EXPIRE = 60 * 60 * 1L;
    /**  过期时长为6小时，单位：秒 */
    public final static long HOUR_SIX_EXPIRE = 60 * 60 * 6L;

    /**  默认过期时长为7天，单位：秒 */
    public final static long Day_One_EXPIRE =  60 * 60 * 24L;

    /**  默认过期时长为7天，单位：秒 */
    public final static long Day_Seven_EXPIRE = 7 * 60 * 60 * 24L;
    /**  默认过期时长为15天，单位：秒 */
    public final static long Day_Fifteen_EXPIRE = 15 * 60 * 60 * 24L;
    /**  默认过期时长为30天，单位：秒 */
    public final static long Day_Thirty_EXPIRE = 30 * 60 * 60 * 24L;


    /**  不设置过期时长 */
    public final static long NOT_EXPIRE = -1L;


    /**
     *  key:val set
     * */
    public void set(String key, Object value, long expire){
        redisTemplate.opsForValue().set(key, value);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
    }

    /**
     *  key:val set
     * */
    public void set(String key, Object value){
        set(key, value, DEFAULT_EXPIRE);
    }

    /**
     *  key:val 多个set
     * */
    public void multiSet(Map<String, String> map) {
        if(CollectionUtils.isEmpty(map)) {
            return ;
        }
        redisTemplate.opsForValue().multiSet(map);
    }

    /**
     *  key:val 多个set
     * */
    public void multiSet(Map<String, String> map, long expire) {
        if(CollectionUtils.isEmpty(map)) {
            return ;
        }
        redisTemplate.opsForValue().multiSet(map);
        if (expire != NOT_EXPIRE) {
            for(String key:map.keySet()) {
                redisTemplate.expire(key, expire, TimeUnit.SECONDS);
            }
        }
    }

    /**
     *  key:val get
     * */
    public Object get(String key, long expire) {
        Object value = redisTemplate.opsForValue().get(key);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
        return value;
    }

    /**
     *  key:val get
     * */
    public Object get(String key) {
        return get(key, NOT_EXPIRE);
    }

    /**
     *  key:val get list
     * */
    public List<Object> getBatchList(Set<String> keys) {
        return redisTemplate.opsForValue().multiGet(keys);
    }


    /**
     * key:val get map
     * 获取多个key的缓存值，返回对象中不包含未空的缓存值
     * */
    @Deprecated
    public Map<String, Object> getBatchMap(Set<String> keys) {
        List<Object> values = getBatchList(keys);
        Map<String, Object> result = new LinkedHashMap<>();
        if (!CollectionUtils.isEmpty(keys)) {
            int index = 0;
            for (String key : keys) {
                Object val=values.get(index);
                if ( val != null) {
                    result.put(key, val);
                }
                index++;
            }
        }
        return result;
    }


    /**
     * 是否存在key
     * */
    public boolean hasKey(String key) {
        return redisTemplate.hasKey(key);
    }


    /**
     *  key:val del
     * */
    public void delete(String key) {
        redisTemplate.delete(key);
    }
    /**
     *  key:val dels
     * */
    public void delete(Collection<String> keys) {
        redisTemplate.delete(keys);
    }

    /**
     *  [hash] get
     * */
    public Object hGet(String key, String field) {
        return redisTemplate.opsForHash().get(key, field);
    }


    /**
     *  [hash] get
     * */
    public List<Object> hMGet(String key, List<Object> fields) {
        return redisTemplate.opsForHash().multiGet(key, fields);
    }


    /**
     *  [hash] get
     * */
    public Map<String, Object> hGetAll(String key){
        HashOperations<String, String, Object> hashOperations = redisTemplate.opsForHash();
        return hashOperations.entries(key);
    }

    /**
     *  [hash] set
     * */
    public void hMSet(String key, Map<String, Object> map){
        hMSet(key, map, DEFAULT_EXPIRE);
    }

    /**
     *  [hash] set
     * */
    public void hMSet(String key, Map<String, Object> map, long expire){
        redisTemplate.opsForHash().putAll(key, map);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
    }
    /**
     *  [hash] set
     * */
    public void hSet(String key, String field, Object value) {
        hSet(key, field, value, DEFAULT_EXPIRE);
    }

    /**
     *  [hash] set
     * */
    public void hSet(String key, String field, Object value, long expire) {
        redisTemplate.opsForHash().put(key, field, value);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
    }

    /**
     *  [hash] is hashKey
     * */
    public boolean hHasKey(String key,String hashKey) {
        return  redisTemplate.opsForHash().hasKey(key, hashKey);
    }


    /**
     *  [hash 散列] del
     * */
    public void hDel(String key, Object... fields){
        redisTemplate.opsForHash().delete(key, fields);
    }



    /**
     * [Set 集合] set
     * */
    public Long sSet(String key,Collection<Object> values ){
        Object[] values2= values.stream().toArray(Object[]::new);
        return this.sSet(key,values2);
    }


    public Long sSet_str(String key,Collection<String> values ){
        Object[] values2= values.stream().toArray(Object[]::new);
        return this.sSet(key,values2);
    }

    /**
     * [Set 集合] set
     * */
    public Long sSet(String key,Collection<Object> values,long expire){
        Object[] values2= values.stream().toArray(Object[]::new);
        return this.sSet(expire,key,values2);
    }

    public Long sSet_str(String key,Collection<String> values,long expire){
        Object[] values2= values.stream().toArray(Object[]::new);
        return this.sSet(expire,key,values2);
    }

    /**
     * [Set 集合] set
     * */
    public Long sSet(String key,Object ... values ){
        return redisTemplate.opsForSet().add(key,values);
    }

    /**
     * [Set 集合] set
     * */
    public Long sSet(long expire, String key,Object ... values ){
        Long l= redisTemplate.opsForSet().add(key,values);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
        return l;
    }

    /**
     * [Set 集合] get
     * */
    public Set<Object> sGet(String key){
        return redisTemplate.opsForSet().members(key);
    }

    /**
     * [Set 集合] remove
     * */
    public Long sRemove(String key,Object ... value){
         return redisTemplate.opsForSet().remove(key,value);
    }

    /**
     * [Set 集合] 是否存在值
     * */
    public boolean sIsValue(String key,Object value){
        return redisTemplate.opsForSet().isMember(key,value);
    }

    /**
     * [Set 集合] 是否存在值
     * */
    public Map<Object, Boolean> sIsValue(String key,Object... values){
        return redisTemplate.opsForSet().isMember(key,values);
    }

    /**
     * [Set 集合] size
     * */
    public Long sSize(String key){
        return redisTemplate.opsForSet().size(key);
    }




    /**
     * [Zset 有序集合] size
     * */
    public Long zsSize(String key){
        return redisTemplate.opsForZSet().size(key);
    }



    /**
     * [List 列表] size
     * */
    public Long lAdd(String key,Object ... values){
        return redisTemplate.opsForList().rightPushAll(key,values);
    }




    /**
     * [List 列表] size
     * */
    public Long lSize(String key){
        return redisTemplate.opsForList().size(key);
    }

    /**
     * [list 列表] push
     * */
    public void leftPush(String key, Object value){
        leftPush(key, value, DEFAULT_EXPIRE);
    }

    /**
     * [List 列表] push
     * */
    public void leftPush(String key, Object value, long expire){
        redisTemplate.opsForList().leftPush(key, value);
        if(expire != NOT_EXPIRE){
            expire(key, expire);
        }
    }

    /**
     * [List 列表] pop
     * */
    public Object rightPop(String key){
        return redisTemplate.opsForList().rightPop(key);
    }



    /**
     *
     * */
    public Set<String> keys(String pattern){
        return redisTemplate.keys(pattern);
    }

    /**
     *
     * */
    public void expire(String key, long expire){
        redisTemplate.expire(key, expire, TimeUnit.SECONDS);
    }




    /**原子操作 : 增 */
    public Long increment(String key,long delta) {
        return redisTemplate.opsForValue().increment(key,delta);
    }
    /**原子操作 : 增 */
    public Long increment(String key) {
        return redisTemplate.opsForValue().increment(key,1);
    }
    /**原子操作 : 减 */
    public Long decrement(String key,long delta) {
        return redisTemplate.opsForValue().decrement(key, delta);
    }
    /**原子操作 : 减 */
    public Long decrement(String key) {
        return redisTemplate.opsForValue().decrement(key);
    }



    /**MQ[基于List的 LPUSH+BRPOP 的实现]:*/
    public void produce_MQList(String key, String message) {
        redisTemplate.opsForList().rightPush(key, message);
    }

    /**MQ[基于List的 LPUSH+BRPOP 的实现]:*/
    public void consume_MQList(String key, OneMessageListener listener) {
        while (true) {
            String msg = (String) redisTemplate.opsForList().leftPop(key);
            if(listener!=null){
                listener.onMessage(msg);
            }
        }
    }


    /**MQ[基于List的 LPUSH+BRPOP 的实现]:*/
    public void blockingConsume_MQList(String key ,int interval, MultipleMessageListener listener) {
        while (true) {
            List<Object> messageList = redisTemplate.executePipelined(new RedisCallback<Object>() {
                @Override
                public Object doInRedis(RedisConnection connection) throws DataAccessException {
                    //队列没有元素会阻塞操作，直到队列获取新的元素或超时，5表示如果没元素就每五秒去拿一次消息
                    return connection.bRPop(interval, key.getBytes());
                }
            }, new StringRedisSerializer());
            if(listener!=null){
                listener.onMessage(messageList);
            }
        }
    }




}