/**
 * Copyright (c) 2018 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xxh.cloud.framework.common.tree;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.tree.TreeNode;
import com.xxh.cloud.framework.common.validator.AssertUtils;

import java.util.*;

/**
 * 树形结构工具类，如：菜单、部门等
 */
public class TreeUtils {

    /**
     * 根据pid，构建树节点
     */
    public static <T extends TreeNode> List<T> build(List<T> treeNodes, String pid) {
        //pid不能为空
        AssertUtils.isNull(pid, "pid");

        List<T> treeList = new ArrayList<>();
        for(T treeNode : treeNodes) {
            if (pid.equals(treeNode.getPid())) {
                treeList.add(findChildren(treeNodes, treeNode));
            }
        }

        return treeList;
    }

    /**
     * 查找子节点
     */
    private static <T extends TreeNode> T findChildren(List<T> treeNodes, T rootNode) {
        for(T treeNode : treeNodes) {
            if(rootNode.getId().equals(treeNode.getPid())) {
                rootNode.getChildren().add(findChildren(treeNodes, treeNode));
            }
        }
        return rootNode;
    }

    /**
     * 构建树节点
     */
    public static <T extends TreeNode> List<T> build(List<T> treeNodes) {
        List<T> result = new ArrayList<>();

        //list转map
        Map<String, T> nodeMap = new LinkedHashMap<>(treeNodes.size());
        for(T treeNode : treeNodes){
            nodeMap.put(treeNode.getId(), treeNode);
        }

        for(T node : nodeMap.values()) {
            T parent = nodeMap.get(node.getPid());
            if(parent != null && !(node.getId().equals(parent.getId()))){
                parent.getChildren().add(node);
                continue;
            }

            result.add(node);
        }

        return result;
    }



    //----------------------------------------------------------------------------------------

    /**
     * 查找父节点
     */
    private static <T extends TreeNode> List<T>  findParent(List<T> result , Map<String, T> nodeMap, T leafNode) {
        if(StrUtil.isNotEmpty(leafNode.getPid()) ){
            T parent =  nodeMap.get(leafNode.getPid());
            if(parent != null){
                result.add(parent);
                findParent(result,nodeMap,parent);
            }
        }
        return result;
    }


    /**
     * 构建树节点
     */
    public static <T extends TreeNode> List<T> buildByMap(Map<String, T> nodeMap) {
        List<T> result = new ArrayList<>();
        for(T node : nodeMap.values()) {
            T parent = nodeMap.get(node.getPid());
            if(parent != null && !(node.getId().equals(parent.getId()))){
                parent.getChildren().add(node);
                continue;
            }

            result.add(node);
        }
        return result;
    }


    public static <T extends TreeNode> List<T> buildByCondition(List<T> treeNodes,TreeUtilsAbs treeUtilsAbs) {
        //list转map
        Map<String, T> nodeMap = new LinkedHashMap<>(treeNodes.size());
        for(T treeNode : treeNodes){
            nodeMap.put(treeNode.getId(), treeNode);
        }

        if(treeUtilsAbs==null){
            return buildByMap(nodeMap);
        }

        Map<String, T> conditionNodeMap =new HashMap<>();
        for(T node : nodeMap.values()) {
            boolean flag= treeUtilsAbs.condition(node);
            if(flag){
                List<T> resultNode=new ArrayList<>();
                resultNode.add(node);
                findParent(resultNode,nodeMap,node);
                resultNode.forEach(item->{
                    conditionNodeMap.put(item.getId(),item);
                });
            }
        }
        if(MapUtil.isEmpty(conditionNodeMap)){
            return new ArrayList<>();
        }
        return buildByMap(conditionNodeMap);

    }




    public static void main(String[] args) {

    }

}